#!/usr/bin/env python

"""convexhull.py

Calculate the convex hull of a set of n 2D-points 
in O(n log n) time.  Taken from Berg et al., 
Computational Geometry, Springer-Verlag, 1997

Dinu C. Gherman
"""

import random

def myDet(p, q, r):
	"""Calc. the determinant of a special matrix, containing three 2D points.

	The sign, "-" or "+", determines the side, right or left, respectivly,
	on which the point r lies, when measured against a directed vector from p to q.
	"""

	# We use Sarrus' Rule to calculate the determinant.
	sum1 = q[0]*r[1] + p[0]*q[1] + r[0]*p[1]
	sum2 = q[0]*p[1] + r[0]*q[1] + p[0]*r[1]
	return sum1 - sum2

def isRightTurn(tripple):
	[p, q, r] = tripple
	d = myDet(p, q, r)
	if d < 0:
		return 1
	return 0

def ch(P):
	# Get a local list copy of the points 
	# and sort them lexically.
	points = map(None, P)
	points.sort()

	# Build upper half.
	upper = [points[0], points[1]]
	for p in points[2:]:
		upper.append(p)
		while len(upper) > 2 and not isRightTurn(upper[-3:]):
			del upper[-2]

	# Build lower half.
	points.reverse()
	lower = [points[0], points[1]]
	for p in points[2:]:
		lower.append(p)
		while len(lower) > 2 and not isRightTurn(lower[-3:]):
			del lower[-2]

	# Remove duplicates.
	del lower[0]
	del lower[-1]

	return tuple(upper + lower)

def makeRandomData(size = 10, addSqr = 0):
	# Fill a square with random points.
	min, max = 0, 100
	P = []
	for i in range(size):
		rand = random.randint
		x = rand(min+1, max-1)
		y = rand(min+1, max-1)
		P.append((x, y))

	# Add some "outmost" corner points.
	if addSqr != 0:
		P = P + [(min, min), (max, max), (min, max), (max, min)]

	return P

def main():
	P = makeRandomData(1000, 1)
	ch_points = ch(P)
	print "CH points:", ch_points

if __name__ == '__main__':
	main()
